#include "libi2c.h"
#include "libwait.h"

unsigned char i2c_adr0;		// 				<i2c_adr2>	<i2c_adr1>	<i2c_adr0>
unsigned char i2c_adr1;		// 256Kbits:	1010_0AAW	0aaa_aaaa	aaaa_aaaa	15bits/ 32Kbytes
unsigned char i2c_adr2;		//   1Mbits:	1010_AAaW	aaaa_aaaa	aaaa_aaaa	17bits/128Kbytes
unsigned char i2c_ctrl;
unsigned char i2c_data;

/*------------------------------------------------------------------------------
 *
 *	I2C MASTER MODE 初期設定
 *
 */
void i2c_init() {
//	TRISC	= 0b00011000;			// Set INPUT
	SSPCON1	= 0b00101000;			// SSPEN / MASTER
	SSPADD	= 0x09;					// 100kHz(4MHz)
	SSPSTAT	= 0b10000000;			// 100kHz
}

void i2c_idle() {
	while((SSPCON2 & 0x1F) | (SSPSTATbits.R_W));
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM バイト書き込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_adr1 = 0x01;
 *	i2c_adr0 = 0x80;
 *	i2c_data = 0x55;
 *	i2c_eeprom_byte_write();
 */
void i2c_eeprom_byte_write() {

	i2c_idle();
	SSPCON2bits.SEN = 1;						// Start condition

	i2c_idle();
	SSPBUF = (i2c_adr2 << 1) + 0;				// Transmit address(0:write)
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_adr1;							// Transmit address
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_adr0;							// Transmit address
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_data;							// Transmit data
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPCON2bits.PEN = 1;						// Stop condition

	waitM(20);									// wait 10ms
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM ランダム読み込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_adr1 = 0x01;
 *	i2c_adr0 = 0x80;
 *	i2c_eeprom_random_read();
 *	i2c_eeprom_cur_read();
 *	x = i2c_data;
 */
void i2c_eeprom_random_read() {

	i2c_idle();
	SSPCON2bits.SEN = 1;						// Start condition

	i2c_idle();
	SSPBUF = (i2c_adr2 << 1) + 0;				// Transmit address(0:write)
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_adr1;							// Transmit address
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_adr0;							// Transmit address
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPCON2bits.PEN = 1;						// Stop condition

//	i2c_idle();
//	SSPCON2bits.RSEN = 1;						// ReStart condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C EEPROM 連続読み込み
 *
 *	i2c_adr2 = I2C_EEPROM_ADR;
 *	i2c_eeprom_cur_read();
 *	x = i2c_data;
 */
void i2c_eeprom_cur_read() {

	i2c_idle();
	SSPCON2bits.SEN = 1;						// Start condition

	i2c_idle();
	SSPBUF = (i2c_adr2 << 1) + 1;				// Transmit address(1:read)
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPCON2bits.RCEN = 1;						// Receive mode enable

	i2c_idle();
	i2c_data = SSPBUF;							// Receive data

	i2c_idle();
	SSPCON2bits.ACKDT = 1;						// NACK
	SSPCON2bits.ACKEN = 1;						// ACK enable

	i2c_idle();
	SSPCON2bits.PEN = 1;						// Stop condition
}

/*------------------------------------------------------------------------------
 *
 *	I2C 液晶ディスプレイへの送信
 *
 *	i2c_adr0 = I2C_LCD_ADR;
 *	i2c_ctrl = RS_HIGH;
 *	i2c_data = 0x01;
 *	i2c_st7032_send();
 */
void i2c_st7032_send() {

	i2c_idle();
	SSPCON2bits.SEN = 1;						// Start condition

	i2c_idle();
	SSPBUF = (i2c_adr0 << 1) + 0;				// Transmit address(0:write)
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_ctrl;							// Transmit control byte
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPBUF = i2c_data;							// Transmit data
	while(SSPCON2bits.ACKSTAT);

	i2c_idle();
	SSPCON2bits.PEN = 1;						// Stop condition
}

