#include "libi2clcd.h"
#include "libi2c.h"
#include "libwait.h"

static char lcd_x, lcd_y;

/*------------------------------------------------------------------------------
 *
 *	液晶初期化
 *
 */
void lcd_init() {

	waitM(81);									// wait 40ms
	i2c_adr0 = I2C_LCD_ADR;
	i2c_ctrl = RS_LOW;
	lcd_out(0x38);								// 8 bit,N=1,5*7dot
	lcd_out(0x39);								// 8 bit,N=1,5*7dot,IS=1
	lcd_out(0x14);								// Internal OSC frequency adjustment
	lcd_out(0x7F);								// Contrast control
	lcd_out(0x53);								// Power/ICON/Contrast control
												//  0b0101IBCC: Booster circuit off -> 5V
	lcd_out(0x6A);								// Follower control
	waitL(2);									// wait 200ms for power stable
	lcd_out(0x38);								// 8 bit,N=1,5*7dot
	lcd_out(0x0C);								// DISPLAY ON
	lcd_out(0x01);								// CLEAR DISPLAY
	waitM(3);									// wait 1.08ms
	lcd_out(0x06);								// ENTRY MODE SET / CURSOR MOVES TO RIGHT
}

/*------------------------------------------------------------------------------
 *
 *	y 行目の先頭アドレスを返す
 *
 */
unsigned char lcd_get_ddadr(char lcd_y) {

	switch(lcd_y) {
	  case 0:									// 1 行目のアドレス
		return(0x80 + 0x00);
	  case 1:									// 2 行目のアドレス
		return(0x80 + 0x40);
	  case 2:									// 3 行目のアドレス
		return(0x80 + 0x00 + 20);
	  case 3:									// 4 行目のアドレス
		return(0x80 + 0x40 + 20);
	}
	return(0);
}

/*------------------------------------------------------------------------------
 *
 *	タイトル文字表示
 *
 */
void lcd_title() {
	char s1[] = LCD_TITLE1;
	char s2[] = LCD_TITLE2;

	lcd_locate(0, 0);
	lcd_puts(s1);
	lcd_locate(0, 1);
	lcd_puts(s2);
	waitL(10);

	lcd_cgpat(1 << 3 | 0, 0b11000);				// (code << 3 | line, pattern)
	lcd_cgpat(1 << 3 | 1, 0b11000);
	lcd_cgpat(1 << 3 | 2, 0b01110);
	lcd_cgpat(1 << 3 | 3, 0b10001);
	lcd_cgpat(1 << 3 | 4, 0b10000);
	lcd_cgpat(1 << 3 | 5, 0b10001);
	lcd_cgpat(1 << 3 | 6, 0b01110);
	lcd_cgpat(1 << 3 | 7, 0b00000);

	lcd_cgpat(2 << 3 | 0, 0b10000);				// (code << 3 | line, pattern)
	lcd_cgpat(2 << 3 | 1, 0b11000);
	lcd_cgpat(2 << 3 | 2, 0b11100);
	lcd_cgpat(2 << 3 | 3, 0b11110);
	lcd_cgpat(2 << 3 | 4, 0b11100);
	lcd_cgpat(2 << 3 | 5, 0b11000);
	lcd_cgpat(2 << 3 | 6, 0b10000);
	lcd_cgpat(2 << 3 | 7, 0b00000);

	lcd_cgpat(3 << 3 | 0, 0b10000);				// (code << 3 | line, pattern)
	lcd_cgpat(3 << 3 | 1, 0b11000);
	lcd_cgpat(3 << 3 | 2, 0b10100);
	lcd_cgpat(3 << 3 | 3, 0b10010);
	lcd_cgpat(3 << 3 | 4, 0b10100);
	lcd_cgpat(3 << 3 | 5, 0b11000);
	lcd_cgpat(3 << 3 | 6, 0b10000);
	lcd_cgpat(3 << 3 | 7, 0b00000);

	lcd_locate(0, 0);
}

/*------------------------------------------------------------------------------
 *
 *	1 文字出力
 *
 */
void lcd_putc(unsigned char c) {

	i2c_adr0 = I2C_LCD_ADR;
	i2c_ctrl = RS_HIGH;
	lcd_out(c);
	lcd_inc_pos();
}

/*------------------------------------------------------------------------------
 *
 *	1 行出力
 *
 */
void lcd_puts(unsigned char *s) {

	unsigned char *p = s;

	while(*p) {
		lcd_putc(*p);
		p++;
	}
}

/*------------------------------------------------------------------------------
 *
 *	move cursor +1
 *
 */
void lcd_inc_pos() {

	if(++lcd_x < LCD_MAX_X) {
		lcd_locate(lcd_x, lcd_y);
		return;
	}
	lcd_x = 0;									// 改行した

	if(++lcd_y < LCD_MAX_Y) {
		lcd_locate(lcd_x, lcd_y);
		return;
	}
	lcd_y = 0;									// 一周した

	lcd_locate(lcd_x, lcd_y);
}

/*------------------------------------------------------------------------------
 *
 *	move_cursor -1
 *
 */
void lcd_dec_pos() {

	if(--lcd_x >= 0) {
		lcd_locate(lcd_x, lcd_y);
		return;
	}
	lcd_x = LCD_MAX_X - 1;

	if(--lcd_y >= 0) {
		lcd_locate(lcd_x, lcd_y);
		return;
	}
	lcd_y = LCD_MAX_Y - 1;

	lcd_locate(lcd_x, lcd_y);
}

/*------------------------------------------------------------------------------
 *
 *	表示クリア
 *
 */
void lcd_clear() {

	i2c_adr0 = I2C_LCD_ADR;
	i2c_ctrl = RS_LOW;
	lcd_out(0x01);								// clear command
		
	waitM(3);									// wait 1.08ms

	lcd_x = lcd_y = 0;
}

/*------------------------------------------------------------------------------
 *
 *	カーソル移動
 *
 */
void lcd_locate(char x, char y) {

	lcd_x = x;
	lcd_y = y;

	i2c_adr0 = I2C_LCD_ADR;
	i2c_ctrl = RS_LOW;
	lcd_out(lcd_get_ddadr(lcd_y) + lcd_x);
}

/*------------------------------------------------------------------------------
 *
 *	パターン定義
 *
 */
void lcd_cgpat(unsigned char adr, char pat) {

	i2c_adr0 = I2C_LCD_ADR;

	i2c_ctrl = RS_LOW;
	lcd_out(0x40 | adr);

	i2c_ctrl = RS_HIGH;
	lcd_out(pat);
}

/*------------------------------------------------------------------------------
 *
 *	LCD バイト出力
 *
 */
void lcd_out(unsigned char c) {

	i2c_data = c;
	i2c_st7032_send();
	waitS(5);									// wait 26.3us (5 x 3 + 13 = 28us 待つ)
}

