#include <sdcc-lib.h>
#include <pic18fregs.h>	// /usr/share/sdcc/include/pic16/pic18f*.h
#include "inline.h"
#include "libwait.h"
#include "libusart.h"
#include "libi2c.h"
#include "libi2clcd.h"

static __code char __at 0x200000 id_location0 = 'A';
static __code char __at 0x200001 id_location1 = 'B';
static __code char __at 0x200002 id_location2 = 'C';
static __code char __at 0x200003 id_location3 = 'D';
static __code char __at 0x200004 id_location4 = 'E';
static __code char __at 0x200005 id_location5 = 'F';
static __code char __at 0x200006 id_location6 = 'G';
static __code char __at 0x200007 id_location7 = 'H';

static __code char __at (__CONFIG1H) configword1h =
	_IESO_OFF_1H &					// Oscillator Switchover mode disabled
	_FCMEN_OFF_1H &					// Fail-Safe Clock Monitor disabled
	_OSC_INTIO67_1H;				// Internal oscillator block, port function on RA6 and RA7

static __code char __at (__CONFIG2L) configword2l =
	_BORV_3_2L &					// Minimum setting
	_BOREN_OFF_2L &					// Brown-out Reset disabled in hardware and software
	_PWRT_OFF_2L;					// PWRT disabled

static __code char __at (__CONFIG2H) configword2h =
	_WDTPS_32768_2H &				// 1:32768
	_WDT_OFF_2H;					// WDT disabled (control is placed on the SWDTEN bit)

static __code char __at (__CONFIG3H) configword3h =
	_MCLRE_OFF_3H &					// RE3 input pin enabled; MCLR disabled
	_LPT1OSC_OFF_3H &				// Timer1 configured for higher power operation
	_PBADEN_ON_3H &					// PORTB<4:0> pins are configured as analog input channels on Reset
	_CCP2MX_PORTC_3H;				// CCP2 input/output is multiplexed with RC1

static __code char __at (__CONFIG4L) configword4l =
	_DEBUG_OFF_4L &					// Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins
	_XINST_OFF_4L &					// Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
	_LVP_OFF_4L &					// Single-Supply ICSP disabled
	_STVREN_ON_4L;					// Stack full/underflow will cause Reset

static __code char __at (__CONFIG5L) configword5l =
	_CP0_OFF_5L &					// Block 0 (000800-001FFFh) not code-protected
	_CP1_OFF_5L;					// Block 1 (002000-003FFFh) not code-protected

static __code char __at (__CONFIG5H) configword5h =
	_CPB_OFF_5H &					// Boot block (000000-0007FFh) not code-protected
	_CPD_OFF_5H;					// Data EEPROM not code-protected

static __code char __at (__CONFIG6L) configword6l =
	_WRT0_OFF_6L &					// Block 0 (000800-001FFFh) not write-protected
	_WRT1_OFF_6L;					// Block 1 (002000-003FFFh) not write-protected

static __code char __at (__CONFIG6H) configword6h =
	_WRTB_OFF_6H &					// Boot block (000000-0007FFh) not write-protected
	_WRTC_OFF_6H &					// Configuration registers (300000-3000FFh) not write-protected
	_WRTD_OFF_6H;					// Data EEPROM not write-protected

static __code char __at (__CONFIG7L) configword7l =
	_EBTR0_OFF_7L &					// Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
	_EBTR1_OFF_7L;					// Block 1 (002000-003FFFh) not protected from table reads executed in other blocks

static __code char __at (__CONFIG7H) configword7h =
	_EBTRB_OFF_7H;					// Boot block (000000-0007FFh) not protected from table reads executed in other blocks

static __code char __at 0xF00000 eedata[] = {'d', 'u', 'm', 'm', 'y', 0};

#define	LED1	PORTAbits.RA0

void isr(void) __interrupt 1 {		// 1: HIGH priority, 2: LOW priority
	INTCONbits.TMR0IF = 0;			// Clear timer interrupt
}

/*			  _______
	NC		-|E3 U	7|-	NC
	LED1	-|A0	6|-	NC
	NC		-|1		5|-	NC
	NC		-|2		4|-	NC
	NC		-|3		3|-	NC
	NC		-|4		2|-	NC
	NC		-|5		1|-	NC
	GND		-|	   B0|-	NC
	NC		-|7		 |-	Vcc
	NC		-|A6	 |-	GND
	NC		-|C0	7|-	RX
	NC		-|1		6|-	TX
	NC		-|2		5|-	NC
	SCL		-|3_____4|-	SDA
*/
/*			  _______
	A0		-|1  U	8|-	Vcc			A0/A1:Address select
	A1		-|2		7|-	WP			WP:GND
	NC		-|3		6|-	SCL
	GND		-|4_____5|-	SDA
*/

void main(void) {

	unsigned char count = 0;

				//  76543210
	PORTA		= 0b00000000;
	LATA		= 0b00000000;
	ADCON1		= 0b00001111;
	CMCON		= 0b00000111;
	TRISA		= 0b00000000;		// 1: 入力, 0: 出力

									//	RA0:AN0			RB2:AN8
									//	RA1:AN1			RB3:AN9
									//	RA2:AN2(Vref-)	RB1:AN10
									//	RA3:AN3(Vref+)	RB4:AN11
									//	RA5:AN4			RB0:AN12

	PORTB		= 0b00000000;
	LATB		= 0b00000000;
	TRISB		= 0b00000000;		// 1: 入力, 0: 出力

	PORTC		= 0b00000000;
	LATC		= 0b00000000;
	TRISC		= 0b11011000;		// 1: 入力, 0: 出力

	INTCON		= 0b00000000;		// 初期値:0b0000000x
									// 7:GIE/GIEH Global Interrupt Enable bit
									// 6:PEIE/GIEL Peripheral Interrupt Enable bit
									// 5 TMR0IE: TMR0 Overflow Interrupt Enable bit
									// 4:INT0IE INT0 External Interrupt Enable bit
									// 3:RBIE RB Port Change Interrupt Enable bit
									// 2:TMR0IF TMR0 Overflow Interrupt Flag bit
									// 1:INT0IF INT0 External Interrupt Flag bit
									// 0:RBIF RB Port Change Interrupt Flag bit

//	INTCON2		= 0b11110101;		// 初期値:0b11110101
									// 7:RBPU PORTB Pull-up Enable bit(1:Enable)
									// 6:INTEDG0 External Interrupt 0 Edge Select bit(1:Rising)
									// 5:INTEDG1 External Interrupt 1 Edge Select bit(1:Rising)
									// 4:INTEDG2 External Interrupt 2 Edge Select bit(1:Rising)
									// 2:TMR0IP TMR0 Overflow Interrupt Priority bit(1:High)
									// 0:RBIP RB Port Change Interrupt Priority bit(1:High)

//	INTCON3		= 0b11000000;		// 初期値:0b11000000
									// 7:INT2IP INT2 External Interrupt Priority bit(1:High)
									// 6:INT1IP INT1 External Interrupt Priority bit(1:High)
									// 4:INT2IE INT2 External Interrupt Enable bit(1:Enable)
									// 3:INT1IE INT1 External Interrupt Enable bit(1:Enable)
									// 1:INT2IF INT2 External Interrupt Flag bit
									// 0:INT1IF INT1 External Interrupt Flag bit

//	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
//	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
//	IPR1		= 0b11111111;		// 初期値:0b11111111(プライオリティ)
									// 7:PSPIF Parallel Slave Port Read/Write Interrupt Flag bit
									// 6:ADIF A/D Converter Interrupt Flag bit
									// 5:RCIF EUSART Receive Interrupt Flag bit
									// 4:TXIF EUSART Transmit Interrupt Flag bit
									// 3:SSPIF Master Synchronous Serial Port Interrupt Flag bit
									// 2:CCP1IF CCP1 Interrupt Flag bit
									// 1:TMR2IF TMR2 to PR2 Match Interrupt Flag bit
									// 0:TMR1IF TMR1 Overflow Interrupt Flag bit

//	PIR2		= 0b00000000;		// 初期値:0b00000000(フラグ)
//	PIE2		= 0b00000000;		// 初期値:0b00000000(イネーブル)
//	IPR2		= 0b11111111;		// 初期値:0b11111111(プライオリティ)
									// 7:OSCFIF: Oscillator Fail Interrupt Flag bit
									// 6:CMIF: Comparator Interrupt Flag bit
									// 4:EEIF: Data EEPROM/Flash Write Operation Interrupt Flag bit
									// 3:BCLIF: Bus Collision Interrupt Flag bit
									// 2:HLVDIF: High/Low-Voltage Detect Interrupt Flag bit
									// 1:TMR3IF: TMR3 Overflow Interrupt Flag bit
									// 0:CCP2IF: CCP2 Interrupt Flag bit

//	T0CON		= 0b11111111;		// 初期値:0b11111111
									// 7:TMR0ON Timer0 On/Off Control bit(1:Enable)
									// 6:T08BIT: Timer0 8-Bit/16-Bit Control bit(1:8-bit)
									// 5:T0CS: Timer0 Clock Source Select bit(0:Internal)
									// 4:T0SE: Timer0 Source Edge Select bit(1:High-to-low)
									// 3:PSA: Timer0 Prescaler Assignment bit(0:Assign)
									// 2~0:T0PS Timer0 Prescaler Select bits(000:1/2, 111:1/256)

//	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 7:RD16 16-Bit Read/Write Mode Enable bit(1:Enable)
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Select bit(0:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

//	T2CON		= 0b00000000;		// 初期値:0b00000000
									// 6~3:T2OUTPS Timer2 Output Postscale Select bits(0000:1/1, 0011:1/4, 1111:1/16)
									// 2:TMR2ON Timer2 On bit(1:On)
									// 1~0:T2CKPS Timer2 Clock Prescale Select bits(00:1/1, 01:1/4, 1x:1/16)

	OSCCON		= 0b01100000;		// change to 4MHz
	while(!OSCCONbits.IOFS);

//	forCalibOsc();
	initUsart();
	i2c_init();

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//
	for(count = 0; count < 0x30; ++count) {		// 書き込みテスト
		i2c_adr2 = I2C_EEPROM_ADR;
		i2c_adr1 = 0;
		i2c_adr0 = count;
		i2c_data = count + 0x30;
		i2c_eeprom_byte_write();
		transmitUsart(i2c_data);
	}

	count = 0x10;								// 読み込みテスト
	i2c_adr2 = I2C_EEPROM_ADR;
	i2c_adr1 = 0;
	i2c_adr0 = count;
	i2c_eeprom_random_read();

	lcd_init();									// LCD 表示テスト
	lcd_clear();
	lcd_title();
	waitL(20);

	lcd_puts("TEST1234test5678");
	waitL(20);

	lcd_clear();
	for(count = 1; count < 0x20; ++count) {
		lcd_locate(count & 7, (count >> 3) & 1);

		i2c_adr2 = I2C_EEPROM_ADR;
		i2c_eeprom_cur_read();

		lcd_putc(i2c_data);
		waitL(2);
	}

	while(1);
}

